<?php declare(strict_types=1);

namespace Everyware\Concepts;

use Everyware\Concepts\Contracts\ApiResponse;
use Everyware\Concepts\Exceptions\ConceptDeleteError;
use Everyware\Concepts\Exceptions\ConceptCreateError;
use Everyware\Concepts\Exceptions\ConceptUpdateError;
use Everyware\Concepts\Exceptions\InvalidConceptData;
use Exception;

/**
 * Class AjaxResponse
 * @package Everyware\Concepts
 */
class ConceptApiResponse implements ApiResponse
{
    public static $messages = [
        'alreadyExists' => 'Concept with uuid: %s already exists!',
        'notFoundInDb' => 'Concept with uuid: %s could not be found!',
        'notFoundInOC' => 'Concept with uuid: %s could not be found in Source and could there for not be created in Everyware!',
        'routeNotFound' => 'The requested route: %s dose not exist!',
        'creationSuccess' => 'Concept with uuid %s was created!',
        'deletionSuccess' => 'Concept with uuid %s was deleted!',
        'updateSuccess' => 'Concept with uuid %s was updated!',
        'internalError' => 'Oops, something went wrong on our side. The request is probably valid but needs to be requested again later.',
    ];

    private $response;
    private $statusCode;
    private $message;

    public function __construct($response, $statusCode = null)
    {
        $this->response = $response;
        $this->statusCode = $statusCode;
    }

    public static function alreadyExists($uuid): ConceptApiResponse
    {
        return static::error([
            'message' => sprintf(static::$messages['alreadyExists'], $uuid)
        ], 409);
    }

    public static function routeNotFound($route): ConceptApiResponse
    {
        return static::error([
            'message' => sprintf(static::$messages['routeNotFound'], $route)
        ], 400);
    }

    public static function notFoundInDb($uuid): ConceptApiResponse
    {
        return static::error([
            'message' => sprintf(static::$messages['notFoundInDb'], $uuid)
        ], 404);
    }

    public static function notFoundInOC($uuid): ConceptApiResponse
    {
        return static::error([
            'message' => sprintf(static::$messages['notFoundInOC'], $uuid)
        ], 500);
    }

    public static function internalError(Exception $e): ConceptApiResponse
    {
        return static::error([
            'message' => static::isPublicException($e) ? $e->getMessage() : static::$messages['internalError'],
        ], 500);
    }

    /**
     * Determine if an exception is meant for public use
     *
     * @param Exception $e
     *
     * @return bool
     */
    public static function isPublicException(Exception $e): bool
    {
        return $e instanceof ConceptCreateError ||
            $e instanceof ConceptDeleteError ||
            $e instanceof ConceptUpdateError ||
            $e instanceof InvalidConceptData;
    }

    // Success responses
    // ======================================================

    public static function creationSuccess($uuid): ConceptApiResponse
    {
        return static::success([
            'message' => sprintf(static::$messages['creationSuccess'], $uuid)
        ], 201);
    }

    public static function deletionSuccess($uuid): ConceptApiResponse
    {
        return static::success([
            'message' => sprintf(static::$messages['deletionSuccess'], $uuid)
        ], 201);
    }

    public static function updateSuccess($uuid): ConceptApiResponse
    {
        return static::success([
            'message' => sprintf(static::$messages['updateSuccess'], $uuid)
        ], 201);
    }

    public static function success($response, $statusCode = null): ConceptApiResponse
    {
        return new static(array_replace([
            'success' => true,
        ], $response), $statusCode);
    }

    public static function error($response, $statusCode = null): ConceptApiResponse
    {
        return new static(array_replace([
            'success' => false,
        ], $response), $statusCode);
    }

    /**
     * Send json response
     */
    public function send(): void
    {
        wp_send_json($this->response, $this->statusCode);
    }

    public function setMessage(string $message): void
    {
        $this->message = $message;
    }

    public function setStatusCode(int $statusCode): void
    {
        $this->statusCode = $statusCode;
    }
}
