<?php declare(strict_types=1);

namespace Everyware\Concepts;

use Everyware\Concepts\Contracts\ConceptProvider;
use Everyware\Concepts\Contracts\ConceptRepository;
use Everyware\Concepts\Exceptions\ConceptDeleteError;
use Everyware\Concepts\Exceptions\ConceptCreateError;
use Everyware\Concepts\Exceptions\ConceptUpdateError;
use Everyware\Concepts\Exceptions\InvalidConceptData;
use OcObject;

/**
 * Class AjaxController
 * @package Everyware\Concepts
 */
class ConceptController
{
    /**
     * @var ConceptRepository
     */
    private $repository;

    /**
     * @var ConceptProvider
     */
    private $provider;

    public function __construct(ConceptRepository $repository, ConceptProvider $provider)
    {
        $this->repository = $repository;
        $this->provider = $provider;
    }

    public function create($uuid): ConceptApiResponse
    {
        $post = $this->repository->findByUuid($uuid);

        if ($post instanceof ConceptPost) {
            return ConceptApiResponse::alreadyExists($uuid);
        }

        try {

            $concept = $this->fetchConcept($uuid);

            if ( ! $concept instanceof OcConcept) {
                return ConceptApiResponse::notFoundInOC($uuid);
            }

            if ($concept->hasParent()) {
                $this->createParent($concept);
            }

            $this->repository->create($concept);

            return ConceptApiResponse::creationSuccess($uuid);

        } catch (ConceptCreateError $e) {
            return ConceptApiResponse::internalError($e);
        } catch (InvalidConceptData $e) {
            return ConceptApiResponse::internalError($e);
        }
    }

    /**
     * @param $uuid
     *
     * @return ConceptApiResponse
     */
    public function show($uuid): ConceptApiResponse
    {
        $post = $this->repository->findByUuid($uuid);

        if ( ! $post instanceof ConceptPost) {
            return ConceptApiResponse::notFoundInDb($uuid);
        }

        return ConceptApiResponse::success([
            'data' => $this->extractPostData($post, $uuid)
        ]);

    }

    /**
     * @param $uuid
     *
     * @return ConceptApiResponse
     */
    public function update($uuid): ConceptApiResponse
    {
        $post = $this->repository->findByUuid($uuid);

        if ( ! $post instanceof ConceptPost) {
            return ConceptApiResponse::notFoundInDb($uuid);
        }

        try {

            $concept = $this->fetchConcept($uuid);

            if ( ! $concept instanceof OcConcept) {
                return ConceptApiResponse::notFoundInOC($uuid);
            }

            if ($concept->hasParent()) {
                $this->createParent($concept);
            }

            $this->repository->updateByOcConcept($concept);

            return ConceptApiResponse::updateSuccess($uuid);

        } catch (ConceptUpdateError $e) {
            return ConceptApiResponse::internalError($e);
        } catch (InvalidConceptData $e) {
            return ConceptApiResponse::internalError($e);
        }
    }

    public function remove($uuid): ConceptApiResponse
    {
        $post = $this->repository->findByUuid($uuid);

        if ( ! $post instanceof ConceptPost) {
            return ConceptApiResponse::notFoundInDb($uuid);
        }

        try {

            $this->repository->delete($post);

            return ConceptApiResponse::deletionSuccess($uuid);

        } catch (ConceptDeleteError $e) {
            return ConceptApiResponse::internalError($e);
        }
    }

    /**
     * @param string $uuid
     *
     * @return OcConcept
     * @throws InvalidConceptData
     */
    private function fetchConcept($uuid): ?OcConcept
    {
        return $this->provider->getSingle($uuid, OcConcept::requiredProperties(1));
    }

    /**
     * @param OcConcept $concept
     */
    private function createParent(OcConcept $concept): void
    {
        $parent = $concept->getParent();

        if ($parent instanceof OcConcept) {
            $this->create($parent->uuid);
        }
    }

    /**
     * @param ConceptPost $post
     * @param string|null $uuid
     *
     * @return array
     */
    private function extractPostData(ConceptPost $post, $uuid = null): array
    {
        $parent = $post->parent();
        $postId = $post->id;

        return [
            'uuid' => $uuid ?? $this->repository->findUuidById($postId),
            'postId' => $postId,
            'permalink' => $post->permalink(),
            'parent' => $parent instanceof ConceptPost ? $this->extractPostData($parent) : null
        ];
    }
}
