<?php

namespace Spec\Everyware\Concepts;

use Everyware\Concepts\ConceptApiResponse;
use Everyware\Concepts\ConceptController;
use Everyware\Concepts\ConceptPost;
use Everyware\Concepts\Contracts\ConceptProvider;
use Everyware\Concepts\Contracts\ConceptRepository;
use Everyware\Concepts\Exceptions\ConceptCreateError;
use Everyware\Concepts\OcConcept;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;

class ConceptControllerSpec extends ObjectBehavior
{
    /**
     * @var ConceptRepository
     */
    private $repository;
    /**
     * @var ConceptProvider
     */
    private $provider;

    public function let(ConceptRepository $repository, ConceptProvider $provider): void
    {
        $this->beConstructedWith($repository, $provider);

        $this->provider = $provider;

        $this->repository = $repository;
    }

    public function it_is_initializable(): void
    {
        $this->shouldHaveType(ConceptController::class);
    }

    public function it_will_respond_if_concept_already_exists_during_create(ConceptPost $post): void
    {
        $uuid = 'uuid';

        $this->willTryToFetchPost($uuid, $post);

        $this->create($uuid)->shouldBeLike(ConceptApiResponse::alreadyExists($uuid));
    }

    public function it_will_respond_if_no_concept_can_be_found_from_Open_Content_during_create(): void
    {
        $uuid = 'uuid';

        $this->willTryToFetchPost($uuid, null);

        $this->willTryToFetchConceptFromOC($uuid, null);

        $this->create($uuid)->shouldBeLike(ConceptApiResponse::notFoundInOC($uuid));
    }

    public function it_will_respond_with_success_if_concept_is_created(OcConcept $concept, ConceptPost $post): void
    {
        $uuid = 'uuid_to_create';

        $this->willTryToFetchPost($uuid, null);

        $this->willTryToFetchConceptFromOC($uuid, $concept);

        $concept->hasParent()->shouldBeCalled()->willReturn(false);

        $this->repository
            ->create($concept)
            ->shouldBeCalled()
            ->willReturn($post);

        $this->create($uuid)->shouldBeLike(ConceptApiResponse::creationSuccess($uuid));
    }

    public function it_will_respond_if_no_concept_can_be_found_from_database_during_show(): void
    {
        $uuid = 'uuid';

        $this->willTryToFetchPost($uuid, null);

        $this->show($uuid)->shouldBeLike(ConceptApiResponse::notFoundInDb($uuid));
    }

    public function it_will_respond_with_post_data_if_requested(ConceptPost $post): void
    {
        $uuid = 'uuid';

        $data = [
            'uuid' => $uuid,
            'postId' => 'id',
            'permalink' => 'url',
            'parent' => null
        ];

        $this->willTryToFetchPost($uuid, $post);

        $post
            ->parent()
            ->shouldBeCalled()
            ->willReturn(null);

        $this->willExtractPostData($post, $data, true);

        $this->show($uuid)->shouldBeLike(ConceptApiResponse::success([
            'data' => $data
        ]));
    }

    public function it_will_respond_with_post_data_for_parent_if_nested_object(
        ConceptPost $post,
        ConceptPost $parent
    ): void {
        $uuid = 'uuid';

        $parentData = [
            'uuid' => 'parentUuid',
            'postId' => 'parentId',
            'permalink' => 'parentUrl',
            'parent' => null
        ];

        $data = [
            'uuid' => $uuid,
            'postId' => 'id',
            'permalink' => 'url',
            'parent' => $parentData
        ];

        $this->willTryToFetchPost($uuid, $post);

        $post
            ->parent()
            ->shouldBeCalled()
            ->willReturn($parent);

        $this->willExtractPostData($post, $data, true);

        $parent
            ->parent()
            ->shouldBeCalled()
            ->willReturn(null);

        $this->willExtractPostData($parent, $parentData);

        $this->show($uuid)->shouldBeLike(ConceptApiResponse::success([
            'data' => $data
        ]));
    }

    public function it_will_respond_if_no_concept_can_be_found_from_database_during_update(): void
    {
        $uuid = 'uuid';

        $this->willTryToFetchPost($uuid, null);

        $this->update($uuid)->shouldBeLike(ConceptApiResponse::notFoundInDb($uuid));
    }

    public function it_will_respond_if_no_concept_can_be_found_from_Open_Content_during_update(ConceptPost $post): void
    {
        $uuid = 'uuid';

        $this->willTryToFetchPost($uuid, $post);

        $this->willTryToFetchConceptFromOC($uuid, null);

        $this->update($uuid)->shouldBeLike(ConceptApiResponse::notFoundInOC($uuid));
    }

    public function it_will_respond_with_success_if_concept_is_updated(OcConcept $concept, ConceptPost $post): void
    {
        $uuid = 'uuid';

        $this->willTryToFetchPost($uuid, $post);

        $this->willTryToFetchConceptFromOC($uuid, $concept);

        $concept->hasParent()->shouldBeCalled()->willReturn(false);

        $this->repository
            ->updateByOcConcept($concept)
            ->shouldBeCalled()
            ->willReturn($post);

        $this->update($uuid)->shouldBeLike(ConceptApiResponse::updateSuccess($uuid));
    }

    public function it_will_respond_if_no_concept_can_be_found_from_database_during_remove(): void
    {
        $uuid = 'uuid';

        $this->willTryToFetchPost($uuid, null);

        $this->remove($uuid)->shouldBeLike(ConceptApiResponse::notFoundInDb($uuid));
    }

    public function it_will_respond_with_success_if_concept_is_remove(ConceptPost $post): void
    {
        $uuid = 'uuid';

        $this->willTryToFetchPost($uuid, $post);

        $this->repository
            ->delete($post)
            ->shouldBeCalled()
            ->willReturn($post);

        $this->remove($uuid)->shouldBeLike(ConceptApiResponse::deletionSuccess($uuid));
    }

    // helper functions
    // ======================================================

    private function willTryToFetchPost($uuid, $result)
    {
        $this->repository
            ->findByUuid($uuid)
            ->shouldBeCalled()
            ->willReturn($result);

        return $result;
    }

    private function willTryToFetchConceptFromOC($uuid, $result)
    {
        $this->provider
            ->getSingle($uuid, OcConcept::requiredProperties(1))
            ->shouldBeCalled()
            ->willReturn($result);

        return $result;
    }

    private function willExtractPostData(ConceptPost $post, $returnData, $hasUuid = false)
    {
        $post
            ->get('id')
            ->shouldBeCalled()
            ->willReturn($returnData['postId']);

        $post
            ->permalink()
            ->shouldBeCalled()
            ->willReturn($returnData['permalink']);

        if ( ! $hasUuid) {
            $this->repository
                ->findUuidById($returnData['postId'])
                ->shouldBeCalled()
                ->willReturn($returnData['uuid']);
        }
    }
}
